from typing import Any, List, Optional, Tuple, cast

from openpyxl import load_workbook
from openpyxl.worksheet.worksheet import Worksheet


def get_list_of_tuples_from_excel(
    path_to_file: str, sheet_name: Optional[str] = None
) -> List[Tuple[Any, ...]]:
    workbook = load_workbook(filename=path_to_file)
    if sheet_name:
        sheet = cast(Worksheet, workbook[sheet_name])
    else:
        sheet = workbook.active

    return list(sheet.values)



import logging
from typing import Callable, Iterable, List, Optional, Tuple, cast

from table2sql.converters.from_csv import get_list_of_tuples_from_csv
from table2sql.converters.from_excel import get_list_of_tuples_from_excel

logger = logging.getLogger()

INSERT_TEMPLATE = """
INSERT INTO {table_name} {column_names}
VALUES {values};
""".strip()

TYPES_MAP = {
    "int": int,
    "float": float,
    "str": lambda value: f"'{value}'",
    "sql": lambda sql: sql,
}


def _get_columns_formatted(column_names: Iterable[str]):
    return f"({', '.join(column_names)})"


def _get_values_formatted(values: Iterable[str], types: Optional[List[Callable]]):
    values_ = []
    for row in values:
        if types:
            row = [str(type_(value)) for type_, value in zip(types, row)]
        else:
            row = [str(value) for value in row]
        values_.append(f'({", ".join(row)})')

    return ", ".join(values_)


def _get_insert_statement_formatted(
    table_name: str, column_names_formatted: str, values_formatted: str
):
    return INSERT_TEMPLATE.format(
        table_name=table_name, column_names=column_names_formatted, values=values_formatted
    )


def _get_types_functions(types_str: Tuple[str, ...]):
    types_functions = []
    for type_str in types_str:
        try:

            type_ = TYPES_MAP[type_str]
        except KeyError:
            type_ = str
            logger.warning(
                f"{type_str} was not found in types map. "
                f"Available types: {', '.join(TYPES_MAP.keys())}"
            )
        types_functions.append(type_)
    return types_functions


def _get_file_extension(file_path: str):
    return file_path.split(".")[-1]


def convert_table_file_to_insert_statement(
    path_to_file: str, output_table: str, delimiter=str, has_types_row=bool, sheet_name=None
):
    """Converts CSV file to SQL insert statements.

    If file doesn't contain row with types as second row - every value is treated as string and
    will be available in insert statement with single quotes.

    If file contain row with types and `types_row` argument is set to `True` - types will be used
    to convert columns to given types.

    Args:
        path_to_file (str): Name of file containing data to be converted to SQL insert statements.
        output_table (str): Name of table into which data should be inserted.
        delimiter (str): Delimiter of given CSV file.
        has_types_row (bool, optional): If second row of table file contains row with types
          (from `TYPES_MAP`). Defaults to False.
        sheet_name (str, optional): If file is Excel - pass sheet name which should be converted
          into insert statements.

    Returns:
        str: SQL insert statement
    """

    file_extension = _get_file_extension(path_to_file)

    if file_extension == "csv":
        rows = get_list_of_tuples_from_csv(path_to_file=path_to_file, delimiter=delimiter)
    elif file_extension in ("xlsx", "xlsm", "xltx", "xltm"):
        rows = get_list_of_tuples_from_excel(path_to_file=path_to_file, sheet_name=sheet_name)
    else:
        raise NotImplementedError(f"'.{file_extension}' file extension is not supported")

    types = None
    if has_types_row:
        [column_names, types_str, *values] = rows
        types_str = cast(Tuple[str, ...], types_str)
        types = _get_types_functions(types_str)
    else:
        [column_names, *values] = rows

    column_names_formatted = _get_columns_formatted(column_names)
    values_formatted = _get_values_formatted(values, types=types)

    return _get_insert_statement_formatted(
        table_name=output_table,
        column_names_formatted=column_names_formatted,
        values_formatted=values_formatted,
    )



import logging
from typing import Optional

import click

from .main import convert_table_file_to_insert_statement

logger = logging.getLogger()


@click.command()
@click.argument(
    "path-to-file",
    type=str,
)
@click.option(
    "--output-table",
    required=True,
    type=str,
    help="Name of table to use in SQL insert statement",
)
@click.option("--delimiter", default=",", type=str, help="Delimiter of CSV file", show_default=True)
@click.option(
    "--has-types-row",
    is_flag=True,
    help="If file contains row with types as row 1",
)
@click.option("--output-file", default=None, help="Name of file to write SQL insert to")
def cli(
    path_to_file: str,
    output_table: str,
    delimiter: Optional[str],
    has_types_row: Optional[bool],
    output_file: Optional[str],
):
    """Converts given table file to SQL insert statements."""

    insert_statement = convert_table_file_to_insert_statement(
        path_to_file=path_to_file,
        output_table=output_table,
        delimiter=delimiter,
        has_types_row=has_types_row,
    )

    print(insert_statement)
    if output_file:
        with open(output_file, "w") as f:
            print(f"\nOutput written to {output_file}")
            f.write(insert_statement)



import csv


def get_list_of_tuples_from_csv(path_to_file: str, delimiter=","):
    input_file_ = open(path_to_file)
    return list(tuple(row) for row in csv.reader(input_file_, delimiter=delimiter))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['table2sql', 'table2sql.converters']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0', 'openpyxl>=3.0.7,<4.0.0']

entry_points = \
{'console_scripts': ['table2sql = table2sql.cli:cli']}

setup_kwargs = {
    'name': 'table2sql',
    'version': '1.0.0',
    'description': '',
    'long_description': "# table2sql\n\n[![CI](https://github.com/piotrgredowski/table2sql/actions/workflows/ci.yml/badge.svg)](https://github.com/piotrgredowski/table2sql/actions/workflows/ci.yml)\n[![Quality Gate Status](https://sonarcloud.io/api/project_badges/measure?project=piotrgredowski_table2sql&metric=alert_status)](https://sonarcloud.io/dashboard?id=piotrgredowski_table2sql)\n[![codecov](https://codecov.io/gh/piotrgredowski/table2sql/branch/main/graph/badge.svg?token=fNkIDyWLq7)](https://codecov.io/gh/piotrgredowski/table2sql)\n[![CodeQL](https://github.com/piotrgredowski/table2sql/actions/workflows/codeql-analysis.yml/badge.svg)](https://github.com/piotrgredowski/table2sql/actions/workflows/codeql-analysis.yml)\n\nPython CLI tool which allows you to convert file with table (CSV and Excel) to SQL insert statements.\n\n\n`some.csv`\n```csv\na,b,c,d\nint,str,float,sql\n1,2,3,(SELECT id FROM another.table WHERE name = 'Paul')\n5,6,7,(SELECT id FROM another.table WHERE name = 'Paul')\n```\n\nCommand:\n```bash\ntable2sql some.csv --output-table some.table --has-types-row\n```\n\nResult:\n```sql\nINSERT INTO some.table (a, b, c, d)\nVALUES (1, '2', 3.0, (SELECT id FROM another.table WHERE name = 'Paul')), (5, '6', 7.0, (SELECT id FROM another.table WHERE name = 'Paul'));\n```\n\n## Install\n\n```bash\npip install table2sql\n```\n",
    'author': 'Piotr Gredowski',
    'author_email': 'piotrgredowski@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/piotrgredowski/table2sql',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
